@@ -6,7 +6,7 @@
 ###########################################################################
 
 from __future__ import absolute_import
-import os, logging
+import os, logging, pickle
 import re
 import pysam
 from itertools import groupby, combinations as comb
@@ -38,7 +38,7 @@ class NoDaemonProcess(multiprocessing.Process):
 class MyPool(multiprocessing.pool.Pool):
     Process = NoDaemonProcess
 
-def unknownMods(inputs, out_dir, knownTable, cluster_dict, modTable, misinc_thresh, cov_table, min_cov, tRNA_dict, remap):
+def unknownMods(inputs, knownTable, cluster_dict, modTable, misinc_thresh, cov_table, min_cov, tRNA_dict, remap):
 # find unknown modifications with a total misincorporation threshold >= misinc_thresh
 
 	log.info('Finding potential unannotated mods for {}'.format(inputs))
@@ -57,6 +57,7 @@ def unknownMods(inputs, out_dir, knownTable, cluster_dict, modTable, misinc_thre
 				isodecoder, pos = line.split("\t")[0:2]
 				predRound1[isodecoder].append(int(pos))
 		knownTable = {cluster:[pos for pos in knownTable[cluster] if pos not in predRound1[cluster]] for cluster, pos in knownTable.items()} # subtraction
+		knownTable = defaultdict(list, knownTable)
 	except:
 		next
 
@@ -69,8 +70,8 @@ def unknownMods(inputs, out_dir, knownTable, cluster_dict, modTable, misinc_thre
 		short_isodecoder = "-".join(isodecoder.split("-")[:-1]) if not "chr" in isodecoder else isodecoder
 		anticodon = clusterAnticodon(cons_anticodon, short_isodecoder)
 		for pos in data.keys():
-			cov = cov_table[isodecoder][pos]
-			if (sum(modTable[isodecoder][pos].values()) >= misinc_thresh and cov >= min_cov and pos-1 not in knownTable[cluster]): # misinc above threshold, cov above threshold and not previously known
+			cov = cov_table.loc[(cov_table.isodecoder == isodecoder) & (cov_table.pos == pos), 'cov']
+			if (sum(modTable[isodecoder][pos].values()) >= misinc_thresh and any(cov >= min_cov) and pos-1 not in knownTable[isodecoder]): # misinc above threshold, cov above threshold and not previously known
 				# if one nucleotide dominates misinc. pattern (i.e. >= 0.9 of all misinc, likely a true SNP or misalignment)
 				if (max(modTable[isodecoder][pos].values()) / sum(modTable[isodecoder][pos].values()) > 0.95):
 					# if mod seems to be an inosine (i.e. A with G misinc at 34) add to list and modification SNPs file (see tRNAtools.ModsParser())
@@ -232,7 +233,7 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, del_dict, clu
 	# This reflects the proportion of reads at a given site that stop at this site, as opposed to the proportion of all reads for the reference that stop here
 
 	modTable_prop = {isodecoder: {pos: {
-				group: count / cov[isodecoder][pos]
+				group: count / (cov[isodecoder][pos]) if cov[isodecoder][pos] != 0 else 0
 				  for group, count in data.items() if group in ['A','C','G','T']
 								}
 			for pos, data in values.items()
@@ -241,22 +242,37 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, del_dict, clu
 					}
 
 	stopTable_prop = {isodecoder: {
-			pos: count / geneCov[isodecoder]
+			pos: (count / geneCov[isodecoder]) if geneCov[isodecoder] != 0 else 0
 			for pos, count in values.items()
 						}
 		for isodecoder, values in stopTable.items()
 				}
 
 	readthroughTable = {isodecoder: {
-			pos: 1 - (count / cov[isodecoder][pos]) # 1 - stops gives readthrough
+			pos: (1 - (count / cov[isodecoder][pos])) if cov[isodecoder][pos] != 0 else 0 # 1 - stops gives readthrough
 			for pos, count in values.items()
 						}
 		for isodecoder, values in stopTable.items()
 				}
+
+	# reformat cov_table and save
+	cov_table = pd.DataFrame.from_dict(cov)
+	cov_table['pos'] = cov_table.index
+	cov_table['pos'] = cov_table['pos'].astype(int)
+	cov_table_melt = cov_table.melt(id_vars='pos', var_name='isodecoder', value_name='cov')
+	cov_table_melt.dropna(inplace = True)
+	cov_table_melt['bam'] = inputs
+	cov_table_melt = cov_table_melt[['isodecoder', 'pos', 'bam', 'cov']]
+	cov_table_melt.to_csv(out_dir + inputs.split("/")[-1] + "_coverage.txt", sep = "\t", index = False)
+
+	# convert coverages to proprtion mapped reads if it is a fraction
+	mapped_reads = sum(counts[inputs].values())
+	cov_table_newMods = cov_table_melt.copy()
+	if min_cov < 1:
+		cov_table_newMods['cov'] = cov_table_newMods['cov'].div(mapped_reads)
 				
 	# find unknown mod sites
-	new_mods, new_Inosines = unknownMods(inputs, out_dir, knownTable, cluster_dict, modTable_prop, misinc_thresh, cov, min_cov, tRNA_dict, remap)
-	
+	new_mods, new_Inosines = unknownMods(inputs, knownTable, cluster_dict, modTable_prop, misinc_thresh, cov_table_newMods, min_cov, tRNA_dict, remap)
 	# format and output mods and stops if remap is disabled (i.e. also occurs after round 2 of mapping)
 	if not remap:
 	#	new_mods = {}
@@ -282,15 +298,6 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, del_dict, clu
 		pool.close()
 		pool.join()
 
-		# format cov table
-		cov_table = pd.DataFrame.from_dict(cov)
-		cov_table['pos'] = cov_table.index
-		cov_table['pos'] = cov_table['pos'].astype(int)
-		cov_table_melt = cov_table.melt(id_vars='pos', var_name='isodecoder', value_name='cov')
-		cov_table_melt.dropna(inplace = True)
-		cov_table_melt['bam'] = inputs
-		cov_table_melt = cov_table_melt[['isodecoder', 'pos', 'bam', 'cov']]
-
 		# copy and add NAs for correct merging with modTable
 		cov_table_na = cov_table_melt.copy()
 		names, dfs = splitTable(cov_table_na)
@@ -305,9 +312,6 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, del_dict, clu
 		#modTable_prop_melt = addNA(modTable_prop_melt, tRNA_struct, cluster_dict, "mods")
 		modTable_prop_melt = modTable_prop_melt[['isodecoder','pos', 'type','proportion','condition', 'bam', 'cov']]
 
-		# save
-		cov_table_melt.to_csv(out_dir + inputs.split("/")[-1] + "_coverage.txt", sep = "\t", index = False)
-
 		# reformat stopTable and add gaps 
 		stopTable_prop_df = pd.DataFrame.from_dict(stopTable_prop)
 		stopTable_prop_df['pos'] = stopTable_prop_df.index
@@ -519,19 +523,15 @@ def addNA(tRNA_struct, data_type, name, table):
 	shortname = "-".join(name.split("-")[:-1]) if not "chr" in name else name
 	new = pd.DataFrame()
 	for pos in tRNA_struct.loc[shortname].index:
-		if data_type == 'mods':
-			#new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'type':pd.Categorical(['A','C','G','T']), 'proportion':'NA', 'condition':table.condition.iloc[1], 'bam':table.bam.iloc[1], 'cov':'NA'})
-			new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'type':pd.Categorical(['A','C','G','T']), 'proportion':'NA', 'condition':table.condition.iloc[1], 'bam':table.bam.iloc[1]})
-		elif data_type == 'stops':
-			new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'proportion':'NA', 'condition':table.condition.iloc[0], 'bam':table.bam.iloc[0]}, index=[0])
-		elif data_type == 'cov':
-			new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'bam':table.bam.iloc[0], 'cov':'NA'}, index=[0])
-		if tRNA_struct.loc[(shortname, pos)].struct == 'gap':
-			if not pos == max(tRNA_struct.loc[shortname].index):
-				table.loc[(table.isodecoder == name) & (table.pos >= pos), 'pos'] += 1
-			#if not data_type == "readthrough":
-			table = table.append(new)
 		if not any(table.loc[table.isodecoder == name].pos == pos): #and not (data_type == "readthrough"):
+			if data_type == 'mods':
+				#new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'type':pd.Categorical(['A','C','G','T']), 'proportion':'NA', 'condition':table.condition.iloc[1], 'bam':table.bam.iloc[1], 'cov':'NA'})
+				new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'type':pd.Categorical(['A','C','G','T']), 'proportion':'NA', 'condition':table.condition.iloc[1], 'bam':table.bam.iloc[1]})
+			elif data_type == 'stops':
+				new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'proportion':'NA', 'condition':table.condition.iloc[0], 'bam':table.bam.iloc[0]}, index=[0])
+			elif data_type == 'cov':
+				new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'bam':table.bam.iloc[0], 'cov':'NA'}, index=[0])
+			
 			table = table.append(new)
 
 	return(table)
@@ -574,18 +574,22 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 		multi = len(baminfo)
 
 	# get tRNA struct info from ssAlign
-	tRNA_struct, tRNA_ungap2canon = tRNAclassifier()[0:2]
-	cons_pos_dict = tRNAclassifier()[3]
+	#tRNA_struct, tRNA_ungap2canon = tRNAclassifier()[0:2]
+	tRNA_ungap2canon = tRNAclassifier()[1]
+	#cons_pos_dict = tRNAclassifier()[3]
+	tRNA_struct = tRNAclassifier_nogaps(oneBased=True)
 
 	tRNA_struct_df = pd.DataFrame(tRNA_struct).unstack().rename_axis(('cluster', 'pos')).rename('struct')
 	tRNA_struct_df = pd.DataFrame(tRNA_struct_df)
+	tRNA_struct_df = tRNA_struct_df.loc[~pd.isna(tRNA_struct_df.struct)]
 
-	# format canonical position dictioary for merging with various tables below
+	# format canonical position dictionary for merging with various tables below
 	tRNA_ungap2canon_table = pd.DataFrame.from_dict(tRNA_ungap2canon, orient = "index")
 	tRNA_ungap2canon_table = tRNA_ungap2canon_table.reset_index()
 	tRNA_ungap2canon_table = tRNA_ungap2canon_table.melt(var_name='pos', value_name='canon_pos', id_vars='index')
 	tRNA_ungap2canon_table.columns = ['isodecoder', 'pos', 'canon_pos']
 	tRNA_ungap2canon_table['pos'] = tRNA_ungap2canon_table['pos'].astype(int)
+	tRNA_ungap2canon_table = tRNA_ungap2canon_table[~pd.isna(tRNA_ungap2canon_table.canon_pos)]
 
 	# initiate custom non-daemonic multiprocessing pool and run with bam names
 	pool = MyPool(multi)
@@ -603,6 +607,12 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 
 		# Redo newModsParser here so that knownTable is updated with new mods from second round and written to allModsTable
 		Inosine_clusters, snp_tolerance, newtRNA_dict, newknownTable = newModsParser(out_dir, name, new_mods, new_Inosines, knownTable, Inosine_lists, tRNA_dict, clustering, remap, snp_tolerance = True)
+		# convert newknownTable to DataFrame for merging and canon_pos addition
+		newknownTable_df = pd.DataFrame.from_dict(newknownTable, orient = "index").melt(ignore_index=False).dropna()[['value']].reset_index()
+		newknownTable_df.columns = ['isodecoder', 'pos']
+		newknownTable_df.loc[~newknownTable_df['isodecoder'].str.contains("chr"), 'isodecoder'] = newknownTable_df['isodecoder'].str.split("-").str[:-1].str.join("-")
+		newknownTable_df = pd.merge(newknownTable_df, tRNA_ungap2canon_table, on = ['isodecoder', 'pos'], how = "left")
+		newknownTable_df.to_csv(out_dir + "mods/allModsTable.csv", sep = "\t", index = False, na_rep = 'NA')
 
 		modTable_total = pd.DataFrame()
 		countsTable_total = pd.DataFrame()
@@ -613,6 +623,7 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 		dinuc_table = pd.DataFrame()
 		CCAvsCC_table = pd.DataFrame()
 
+		# generate counts table to select tRNAs to filter
 		for bam in bamlist:
 			countsTable = pd.read_csv(bam + "countTable.csv", header = 0, sep = "\t")
 			os.remove(bam + "countTable.csv")
@@ -621,7 +632,7 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 			else:
 				countsTable_total = pd.merge(countsTable_total, countsTable, on = "isodecoder", how = "left")
 
-		# get isodecoders to filter from mods and stops
+		# get isodecoders to filter
 		countsTable_total.index = countsTable_total.isodecoder
 		countsTable_total.drop(columns = ['isodecoder'], inplace = True)
 		filtered, filter_warning = filterCoverage(countsTable_total, min_cov)
@@ -631,27 +642,38 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 			modTable = pd.read_csv(bam + "mismatchTable.csv", header = 0, sep = "\t")
 			modTable = modTable[~modTable.isodecoder.isin(filtered)]
 			modTable = modTable[~modTable.isodecoder.isin(splitBool)]
-			modTable['canon_pos'] = modTable['pos'].map(cons_pos_dict)
+			modTable.loc[~modTable['isodecoder'].str.contains("chr"), 'isodecoder'] = modTable['isodecoder'].str.split("-").str[:-1].str.join("-")
+			modTable['pos'] = modTable['pos'] - 1 # subtract 1 for 0-based numbering to get canon_pos
+			modTable = pd.merge(modTable, tRNA_ungap2canon_table, on = ['isodecoder', 'pos'], how = "left")
+			modTable['pos'] = modTable['pos'] + 1 # return to 1-based
+			#modTable['canon_pos'] = modTable['pos'].map(cons_pos_dict)
 			# edit misinc. propoportions of inosines to reflect true level of Gs, set As to NA
-			log.info("Editing inosine proportions for {}".format(bam))
 			for cluster in Inosine_clusters:
 				for isodecoder in cluster_dict[cluster]:
-					if any(modTable.isodecoder.str.contains(isodecoder)):
-						modTable.at[(modTable.canon_pos == '34') & (modTable['type'] == 'G') & (modTable.isodecoder == isodecoder), 'proportion'] = 1 - sum(modTable[(modTable.canon_pos == '34') & (modTable['type'] != 'G') & (modTable.isodecoder == isodecoder)]['proportion'].dropna())
-						modTable.at[(modTable.canon_pos == '34') & (modTable['type'] == 'A') & (modTable.isodecoder == isodecoder), 'proportion'] = np.nan
-			log.info("Finished for {}".format(bam))
+					iso_short = "-".join(isodecoder.split("-")[:-1])
+					if any(modTable.isodecoder.str.contains(iso_short)):
+						modTable.at[(modTable.canon_pos == '34') & (modTable['type'] == 'G') & (modTable.isodecoder == iso_short), 'proportion'] = 1 - sum(modTable[(modTable.canon_pos == '34') & (modTable['type'] != 'G') & (modTable.isodecoder == iso_short)]['proportion'].dropna())
+						modTable.at[(modTable.canon_pos == '34') & (modTable['type'] == 'A') & (modTable.isodecoder == iso_short), 'proportion'] = np.nan
 			os.remove(bam + "mismatchTable.csv")
 
 			stopTable = pd.read_csv(bam + "RTstopTable.csv", header = 0, sep = "\t")
 			stopTable = stopTable[~stopTable.isodecoder.isin(filtered)]
 			stopTable = stopTable[~stopTable.isodecoder.isin(splitBool)]
-			stopTable['canon_pos'] = stopTable['pos'].map(cons_pos_dict)
+			stopTable.loc[~stopTable['isodecoder'].str.contains("chr"), 'isodecoder'] = stopTable['isodecoder'].str.split("-").str[:-1].str.join("-")
+			stopTable['pos'] = stopTable['pos'] - 1
+			stopTable = pd.merge(stopTable, tRNA_ungap2canon_table, on = ['isodecoder', 'pos'], how = "left")
+			stopTable['pos'] = stopTable['pos'] + 1
+			#stopTable['canon_pos'] = stopTable['pos'].map(cons_pos_dict)
 			os.remove(bam + "RTstopTable.csv")
 
 			readthroughTable = pd.read_csv(bam + "readthroughTable.csv", header = 0, sep = "\t")
 			readthroughTable = readthroughTable[~readthroughTable.isodecoder.isin(filtered)]
 			readthroughTable = readthroughTable[~readthroughTable.isodecoder.isin(splitBool)]
-			readthroughTable['canon_pos'] = readthroughTable['pos'].map(cons_pos_dict)
+			readthroughTable.loc[~readthroughTable['isodecoder'].str.contains("chr"), 'isodecoder'] = readthroughTable['isodecoder'].str.split("-").str[:-1].str.join("-")
+			readthroughTable['pos'] = readthroughTable['pos'] - 1
+			readthroughTable = pd.merge(readthroughTable, tRNA_ungap2canon_table, on = ['isodecoder', 'pos'], how = "left")
+			readthroughTable['pos'] = readthroughTable['pos'] + 1
+			#readthroughTable['canon_pos'] = readthroughTable['pos'].map(cons_pos_dict)
 			os.remove(bam + "readthroughTable.csv")
 
 			newModsTable = pd.read_csv(bam + "_predictedModstemp.csv", header = None, names = ['isodecoder', 'pos', 'identity', 'bam'], sep = "\t")
@@ -679,21 +701,21 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 
 		# output tables
 		log.info("Output final tables, counts and new mods...")
-		modTable_total.loc[~modTable_total['isodecoder'].str.contains("chr"), 'isodecoder'] = modTable_total['isodecoder'].str.split("-").str[:-1].str.join("-")
 		modTable_total.drop_duplicates(inplace = True)
+		modTable_total.sort_values(by=['isodecoder', 'pos'], inplace = True)
 		modTable_total.to_csv(out_dir + "mods/mismatchTable.csv", sep = "\t", index = False, na_rep = 'NA')
-		with open(out_dir + "mods/allModsTable.csv", "w") as known:
-			known.write("cluster\tpos\n")
-			for cluster, data in newknownTable.items():
-				for pos in data:
-					known.write("-".join(cluster.split("-")[:-1]) + "\t" + str(pos+1) + "\n")
-
-		stopTable_total.loc[~stopTable_total['isodecoder'].str.contains("chr"), 'isodecoder'] = stopTable_total['isodecoder'].str.split("-").str[:-1].str.join("-")
+ 		#with open(out_dir + "mods/allModsTable.csv", "w") as known:
+		#	known.write("cluster\tpos\n")
+		#	for cluster, data in newknownTable.items():
+		#		for pos in data:
+		#			known.write("-".join(cluster.split("-")[:-1]) + "\t" + str(pos+1) + "\n")
+		
 		stopTable_total.drop_duplicates(inplace = True)
+		stopTable_total.sort_values(by=['isodecoder', 'pos'], inplace = True)
 		stopTable_total.to_csv(out_dir + "mods/RTstopTable.csv", sep = "\t", index = False, na_rep = 'NA')
 
-		readthroughTable_total.loc[~readthroughTable_total['isodecoder'].str.contains("chr"), 'isodecoder'] = readthroughTable_total['isodecoder'].str.split("-").str[:-1].str.join("-")
 		readthroughTable_total.drop_duplicates(inplace = True)
+		readthroughTable_total.sort_values(by=['isodecoder', 'pos'], inplace = True)
 		readthroughTable_total.to_csv(out_dir + "mods/readthroughTable.csv", sep = "\t", index = False, na_rep = 'NA')	
 		
 		# add column to counts to indicate complete isodecoder split or not, sizes, and parent
@@ -723,12 +745,13 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 		clusterInfo.index = clusterInfo.isodecoder
 		clusterInfo.drop(columns = ['isodecoder'], inplace = True)
 		countsTable_total = countsTable_total.join(clusterInfo)
-		countsTable_total.to_csv(out_dir + "Isodecoder_counts.txt", sep = "\t", index = True, na_rep = "0")
+		countsTable_total.sort_values(by=["isodecoder"], inplace = True)
+		countsTable_total.to_csv(out_dir + "Isodecoder_counts_raw.txt", sep = "\t", index = True, na_rep = "0")
 
 		# map canon_pos for each isodecoder ungapped pos to newMods
+		newMods_total = newMods_total[~newMods_total.isodecoder.isin(filtered)]
 		newMods_total.loc[~newMods_total['isodecoder'].str.contains("chr"), 'isodecoder'] = newMods_total['isodecoder'].str.split("-").str[:-1].str.join("-")
 		newMods_total = pd.merge(newMods_total, tRNA_ungap2canon_table, on = ['isodecoder', 'pos'], how = "left")
-		newMods_total = newMods_total[~newMods_total.isodecoder.isin(filtered)]
 		# make pivot table from mods and add A, C, G, T misinc. proportions for new mods
 		pivot = modTable_total.pivot_table(index = ['isodecoder', 'bam', 'canon_pos'], columns = 'type', values = 'proportion')
 		pivot = pivot.reset_index()
@@ -747,9 +770,9 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 			CCAvsCC_table.to_csv(out_dir + "CCAanalysis/CCAcounts.csv", sep = "\t", index = False)
 
 		# Anticodon and/or isodecoder counts counts
-		countReads(out_dir + "Isodecoder_counts.txt", out_dir, isodecoder_sizes, clustering, newtRNA_dict, clusterInfo)
+		countReads(out_dir + "Isodecoder_counts_raw.txt", out_dir, isodecoder_sizes, clustering, newtRNA_dict, clusterInfo)
 
-		log.info("** Read counts per isodecoder saved to " + out_dir + "counts/Isodecoder_counts.txt **")
+		log.info("** Read counts per isodecoder saved to " + out_dir + "counts/Isodecoder_counts_raw.txt **")
 
 	return(new_mods, new_Inosines, filtered, filter_warning)
 
