@@ -330,8 +330,8 @@ def modsToSNPIndex(gtRNAdb, tRNAscan_out, mitotRNAs, modifications_table, experi
 	# if clustering is not activated then write full gff and report on total SNPs written 
 	if not cluster:
 		coverage_bed = tRNAbed.name
-		mod_lists = dict()
-		Inosine_lists = dict()
+		mod_lists = defaultdict(list)
+		Inosine_lists = defaultdict(list)
 		with open(out_dir + experiment_name + "_tRNA.gff","w") as tRNAgff, open(out_dir + experiment_name + "isoacceptorInfo.txt","w") as isoacceptorInfo:	
 			isoacceptor_dict = defaultdict(int)
 			isoacceptorInfo.write("Isoacceptor\tsize\n")
@@ -445,8 +445,8 @@ def modsToSNPIndex(gtRNAdb, tRNAscan_out, mitotRNAs, modifications_table, experi
 
 		# read cluster files, get nonredudant set of mod positions of all members of a cluster, create snp_records for writing SNP index
 		cluster_pathlist = Path(temp_dir).glob("**/*_clusters.uc")
-		mod_lists = dict() # stores non-redundant sets of mismatches and mod positions for clusters
-		Inosine_lists = dict() # stores positions of inosines for clusters
+		mod_lists = defaultdict(list) # stores non-redundant sets of mismatches and mod positions for clusters
+		Inosine_lists = defaultdict(list) # stores positions of inosines for clusters
 		snp_records = list()
 		cluster_dict = defaultdict(list) # info about clusters and members
 		mismatch_dict = defaultdict(list) # dictionary of mismatches only (not mod positions - required for exclusion from misincorporation analysis in mmQuant)
@@ -749,7 +749,7 @@ def additionalModsParser(input_species, out_dir):
 	# initialise dictionaries of structure (with and without gapped numbering) and anticodon positions to define canonical mod sites
 	tRNA_struct = tRNAclassifier()[0]
 	cons_pos_dict = tRNAclassifier()[3]
-	tRNA_struct_nogap = tRNAclassifier_nogaps()
+	tRNA_struct_nogap = tRNAclassifier_nogaps(oneBased=False)
 	cons_anticodon = getAnticodon()
 
 	# dictionary storing additional mods per tRNA cluster with corrected positions
@@ -1001,7 +1001,7 @@ def countReads(input_counts, out_dir, isodecoder_sizes, clustering, tRNA_dict, c
 
 	count_anticodon_pd = pd.DataFrame.from_dict(count_dict_anticodon, orient='index')
 	count_anticodon_pd.index.name = 'Anticodon'
-	count_anticodon_pd.to_csv(out_dir + 'Anticodon_counts.txt', sep = '\t')
+	count_anticodon_pd.to_csv(out_dir + 'Anticodon_counts_raw.txt', sep = '\t')
 
 	if not clustering:
 		new_count_isodecoder = defaultdict(lambda: defaultdict(int))
@@ -1028,7 +1028,7 @@ def countReads(input_counts, out_dir, isodecoder_sizes, clustering, tRNA_dict, c
 		count_isodecoder_pd = count_isodecoder_pd.join(clusterInfo)
 		count_isodecoder_pd.to_csv(out_dir + 'Isodecoder_counts.txt', sep = '\t')
 
-	log.info("** Read counts per anticodon saved to " + out_dir + "counts/Anticodon_counts.txt **")
+	log.info("** Read counts per anticodon saved to " + out_dir + "counts/Anticodon_counts_raw.txt **")
 
 def tidyFiles (out_dir, cca):
 	
